<?php

namespace Teknisa\SUP\Modules\Cadastros\Services;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Teknisa\SUP\Util\SupUtilities;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Cmp02600ParticularidadesService {
    
    protected $environment;
    protected $entityManager;
    
    public function __construct(SupUtilities $supUtilities, $environment, $entityManager)
    {
        $this->supUtilities = $supUtilities;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
    }
    
    /********************
     *      INSERTS     *
     * ******************/
    
    public function insertNewProductOrParticularity($row)
    {    
        try {
            $query = "INSERT INTO PARTFILI (CDPRODUTO, CDFILIAL, NRSEQPART, CDTIPPART, CDMARCA, CDFORNECED) VALUES (:CDPRODUTO, :CDFILIAL, :NRSEQPART, :CDTIPPART, :CDMARCA, :CDFORNECED)";
            $param = [
                'CDPRODUTO'  => $row['CDPRODUTO'],
                'CDFILIAL'   => $row['CDFILIAL'],
                'NRSEQPART'  => $row['NRSEQPART'],
                'CDTIPPART'  => $row['CDTIPPART'],
                'CDMARCA'    => $row['CDMARCA'],
                'CDFORNECED' => $row['CDFORNECED']
            ];
            return $this->supUtilities->crud($query, $param);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao inserir período!');
        }
    }
    
    
    /********************
     *      UPDATES     *
     * ******************/
     
     
    public function updateParticularity($row)
    {
        try {
            $query = "UPDATE PARTFILI SET CDPRODUTO = :CDPRODUTO, CDFILIAL = :CDFILIAL, CDTIPPART = :CDTIPPART, CDMARCA = :CDMARCA, CDFORNECED = :CDFORNECED WHERE CDPRODUTO = :P_CDPRODUTO AND CDFILIAL = :P_CDFILIAL AND CDTIPPART = :P_CDTIPPART";
            $params = [
                'CDPRODUTO'   => $row['CDPRODUTO'],
                'CDFILIAL'    => $row['CDFILIAL'],
                'CDTIPPART'   => $row['CDTIPPART'],
                'CDMARCA'     => $row['CDMARCA'],
                'CDFORNECED'  => $row['CDFORNECED'],
                'P_CDPRODUTO' => $row['CDPRODUTO'],
                'P_CDFILIAL'  => $row['CDFILIAL'],
                'P_CDTIPPART' => $row['CDTIPPART']
            ];
            $this->supUtilities->crud($query, $params);
        } catch (Exception $e) {
            throw new \Exception('Erro ao atualizar período!');
        }
    }
    
    
    /********************
     *      DELETES     *
     * ******************/
     
    
    public function deleteProductOrParticularity($row)
    {
        try {
            $query = "DELETE FROM PARTFILI WHERE CDPRODUTO = :P_CDPRODUTO AND CDFILIAL = :P_CDFILIAL AND NRSEQPART = :P_NRSEQPART AND CDTIPPART = :P_CDTIPPART";
            $params = [
                'P_CDPRODUTO' => $row['CDPRODUTO'],
                'P_CDFILIAL'  => $row['CDFILIAL'],
                'P_NRSEQPART' => $row['NRSEQPART'],
                'P_CDTIPPART' => $row['CDTIPPART']
            ];
            $this->supUtilities->crud($query, $params);
        } catch (Exception $e) {
            throw new \Exception('Erro ao deletar unidade!');
        }
    }
    
    
    /********************
     *      SELECTS     *
     * ******************/
     
     
    public function validaProdutoExiste($row)
    {
        $query = "SELECT DISTINCT PARTFILI.CDPRODUTO, PRODUTO.CDARVPROD, PRODUTO.NMPRODUTO, PARTFILI.CDFILIAL, PRODUTO.NRORG FROM MARCA, FORNECEDOR, PRODUTO, PARTFILI WHERE PARTFILI.CDFILIAL = :P_CDFILIAL AND PARTFILI.CDPRODUTO = PRODUTO.CDPRODUTO AND PARTFILI.CDMARCA = MARCA.CDMARCA (+) AND PARTFILI.CDFORNECED = FORNECEDOR.CDFORNECED (+) AND PRODUTO.CDPRODUTO = :P_CDPRODUTO";
        $params = [
            'P_CDFILIAL'  => $row['CDFILIAL'],
            'P_CDPRODUTO' => $row['CDPRODUTO']
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function validaMarcaProduto($row)
    {
        $query = "SELECT MARCA.CDMARCA, MARCA.NMMARCA FROM MARCA WHERE (MARCA.CDMARCA = :P_CDMARCA OR :P_CDMARCA = 'T') AND EXISTS (SELECT 1 FROM MARCPRODCOMP WHERE :P_CDPRODUTO = MARCPRODCOMP.CDPRMARCOMPL AND MARCA.CDMARCA = MARCPRODCOMP.CDMARCA) ORDER BY CDMARCA";
        $params = [
            'P_CDMARCA'   => $row['CDMARCA'],
            'P_CDPRODUTO' => $row['CDPRODUTO']
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getProductInformation($row)
    {
        $query = "SELECT PARTFILI.CDPRODUTO, PRODUTO.CDARVPROD, PRODUTO.NMPRODUTO, PARTFILI.CDFILIAL, PARTFILI.CDTIPPART, PARTFILI.NRSEQPART, PRODUTO.NRORG FROM MARCA, FORNECEDOR, PRODUTO, PARTFILI WHERE PARTFILI.CDFILIAL = :P_CDFILIAL AND PARTFILI.CDPRODUTO = PRODUTO.CDPRODUTO AND PARTFILI.CDMARCA = MARCA.CDMARCA (+) AND PARTFILI.CDFORNECED = FORNECEDOR.CDFORNECED (+) AND PRODUTO.CDPRODUTO = :P_CDPRODUTO";
        $params = [
            'P_CDFILIAL'  => $row['CDFILIAL'],
            'P_CDPRODUTO' => $row['CDPRODUTO']
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getExistingBrandParticularities($row)
    {
        $query = "SELECT DISTINCT PARTFILI.CDPRODUTO, PRODUTO.CDARVPROD, PRODUTO.NMPRODUTO, PARTFILI.CDFILIAL, PARTFILI.CDTIPPART, PRODUTO.NRORG FROM MARCA, FORNECEDOR, PRODUTO, PARTFILI WHERE PARTFILI.CDFILIAL = :P_CDFILIAL AND PARTFILI.CDPRODUTO = PRODUTO.CDPRODUTO AND PARTFILI.CDMARCA = MARCA.CDMARCA (+) AND PARTFILI.CDFORNECED = FORNECEDOR.CDFORNECED (+) AND PRODUTO.CDPRODUTO = :P_CDPRODUTO AND (PARTFILI.CDTIPPART <> :P_CDTIPPART OR PARTFILI.CDTIPPART = '1') AND PARTFILI.CDTIPPART <> '4' AND PARTFILI.CDTIPPART <> '5' AND PARTFILI.CDTIPPART <> '6'";
        $params = [
            'P_CDFILIAL'  => $row['CDFILIAL'],
            'P_CDPRODUTO' => $row['CDPRODUTO'],
            'P_CDTIPPART' => $row['CDTIPPART']
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getExistingProviderParticularities($row)
    {
        $query = "SELECT DISTINCT PARTFILI.CDPRODUTO, PRODUTO.CDARVPROD, PRODUTO.NMPRODUTO, PARTFILI.CDFILIAL, PARTFILI.CDTIPPART, PRODUTO.NRORG FROM MARCA, FORNECEDOR, PRODUTO, PARTFILI WHERE PARTFILI.CDFILIAL = :P_CDFILIAL AND PARTFILI.CDPRODUTO = PRODUTO.CDPRODUTO AND PARTFILI.CDMARCA = MARCA.CDMARCA (+) AND PARTFILI.CDFORNECED = FORNECEDOR.CDFORNECED (+) AND PRODUTO.CDPRODUTO = :P_CDPRODUTO AND ( PARTFILI.CDTIPPART <> :P_CDTIPPART OR PARTFILI.CDTIPPART = '4' ) AND PARTFILI.CDTIPPART <> '1' AND PARTFILI.CDTIPPART <> '2' AND PARTFILI.CDTIPPART <> '3'";
        $params = [
            'P_CDFILIAL'  => $row['CDFILIAL'],
            'P_CDPRODUTO' => $row['CDPRODUTO'],
            'P_CDTIPPART' => $row['CDTIPPART']
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function getExistingBrandProviderParticularities($row)
    {
        $query = "SELECT DISTINCT PARTFILI.CDPRODUTO, PRODUTO.CDARVPROD, PRODUTO.NMPRODUTO, PARTFILI.CDFILIAL, PARTFILI.CDTIPPART, PRODUTO.NRORG FROM MARCA, FORNECEDOR, PRODUTO, PARTFILI WHERE PARTFILI.CDFILIAL = :P_CDFILIAL AND PARTFILI.CDPRODUTO = PRODUTO.CDPRODUTO AND PARTFILI.CDMARCA = MARCA.CDMARCA (+) AND PARTFILI.CDFORNECED = FORNECEDOR.CDFORNECED (+) AND PRODUTO.CDPRODUTO = :P_CDPRODUTO AND ( PARTFILI.CDTIPPART <> :P_CDTIPPART OR PARTFILI.CDTIPPART = '7' )";
        $params = [
            'P_CDFILIAL'  => $row['CDFILIAL'],
            'P_CDPRODUTO' => $row['CDPRODUTO'],
            'P_CDTIPPART' => $row['CDTIPPART']
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function validateBrandExists($row)
    {
        $query = "SELECT DISTINCT PARTFILI.CDPRODUTO, PRODUTO.CDARVPROD, PRODUTO.NMPRODUTO, PARTFILI.CDFILIAL, PARTFILI.CDTIPPART, MARCA.CDMARCA, PRODUTO.NRORG FROM MARCA, FORNECEDOR, PRODUTO, PARTFILI WHERE PARTFILI.CDFILIAL = :P_CDFILIAL AND PARTFILI.CDPRODUTO = PRODUTO.CDPRODUTO AND PARTFILI.CDMARCA = MARCA.CDMARCA (+) AND PARTFILI.CDFORNECED = FORNECEDOR.CDFORNECED (+) AND PRODUTO.CDPRODUTO = :P_CDPRODUTO AND PARTFILI.CDTIPPART = :P_CDTIPPART AND MARCA.CDMARCA = :P_CDMARCA";
        $params = [
            'P_CDFILIAL'  => $row['CDFILIAL'],
            'P_CDPRODUTO' => $row['CDPRODUTO'],
            'P_CDTIPPART' => $row['CDTIPPART'],
            'P_CDMARCA'   => $row['CDMARCA']
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function validateProviderExists($row)
    {
        $query = "SELECT DISTINCT PARTFILI.CDPRODUTO, PRODUTO.CDARVPROD, PRODUTO.NMPRODUTO, PARTFILI.CDFILIAL, PARTFILI.CDTIPPART, MARCA.CDMARCA, PRODUTO.NRORG FROM MARCA, FORNECEDOR, PRODUTO, PARTFILI WHERE PARTFILI.CDFILIAL = :P_CDFILIAL AND PARTFILI.CDPRODUTO = PRODUTO.CDPRODUTO AND PARTFILI.CDMARCA = MARCA.CDMARCA (+) AND PARTFILI.CDFORNECED = FORNECEDOR.CDFORNECED (+) AND PRODUTO.CDPRODUTO = :P_CDPRODUTO AND PARTFILI.CDTIPPART = :P_CDTIPPART AND FORNECEDOR.CDFORNECED = :P_CDFORNECED";
        $params = [
            'P_CDFILIAL'   => $row['CDFILIAL'],
            'P_CDPRODUTO'  => $row['CDPRODUTO'],
            'P_CDTIPPART'  => $row['CDTIPPART'],
            'P_CDFORNECED' => $row['CDFORNECED']
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    public function validateBrandProviderExists($row)
    {
        $query = "SELECT DISTINCT PARTFILI.CDPRODUTO, PRODUTO.CDARVPROD, PRODUTO.NMPRODUTO, PARTFILI.CDFILIAL, PARTFILI.CDTIPPART, MARCA.CDMARCA, PRODUTO.NRORG FROM MARCA, FORNECEDOR, PRODUTO, PARTFILI WHERE PARTFILI.CDFILIAL = :P_CDFILIAL AND PARTFILI.CDPRODUTO = PRODUTO.CDPRODUTO AND PARTFILI.CDMARCA = MARCA.CDMARCA (+) AND PARTFILI.CDFORNECED = FORNECEDOR.CDFORNECED (+) AND PRODUTO.CDPRODUTO = :P_CDPRODUTO AND PARTFILI.CDTIPPART = :P_CDTIPPART AND MARCA.CDMARCA = :P_CDMARCA AND FORNECEDOR.CDFORNECED = :P_CDFORNECED";
        $params = [
            'P_CDFILIAL'   => $row['CDFILIAL'],
            'P_CDPRODUTO'  => $row['CDPRODUTO'],
            'P_CDTIPPART'  => $row['CDTIPPART'],
            'P_CDMARCA'    => $row['CDMARCA'],
            'P_CDFORNECED' => $row['CDFORNECED']
        ];
        return $this->supUtilities->crud($query, $params);
    }
    
    
    /***********************
     *      PROCEDURES     *
     * *********************/
     
       
    public function procedureExcluirPartFili($row)
    {
        try {
            $connection = $this->entityManager->getConnection();
            $procedure = new StoredProcedure($connection, 'EXCLUIR_PARTFILI');
            
            $procedure->addParam(new Param('P_TPRODUTO',     Param::PARAM_INPUT, $row['TPRODUTO']));
            $procedure->addParam(new Param('P_CDPRODINI',    Param::PARAM_INPUT, $row['PRODINI']));
            $procedure->addParam(new Param('P_CDPRODFIN',    Param::PARAM_INPUT, $row['PRODFIN']));
            $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $row['CDFILIAL']));
            $procedure->addParam(new Param('P_CDMARCA',      Param::PARAM_INPUT, $row['CDMARCA']));
            $procedure->addParam(new Param('P_CDFORNECED',   Param::PARAM_INPUT, $row['CDFORNECED']));
            $procedure->addParam(new Param('P_CDTIPPART',    Param::PARAM_INPUT, $row['CDTIPPART']));
            $procedure->addParam(new Param('P_CDGRUPFILI',   Param::PARAM_INPUT, $row['CDGRUPFILI']));
            $procedure->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $row['CDOPERADOR']));
            $procedure->addParam(new Param('P_CDFILIALORIG', Param::PARAM_INPUT, null));
            $procedure->addParam(new Param('P_COMMIT',       Param::PARAM_INPUT, 'S'));
            
            $procedure->execute();
        } catch(Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
        
    }
    
    public function procedureIncluiPartFili($row)
    {
        try {
            $connection = $this->entityManager->getConnection();
            $procedure = new StoredProcedure($connection, 'INCLUI_PARTFILI');
            
            $procedure->addParam(new Param('P_CDPRODINI',    Param::PARAM_INPUT, $row['PRODINI']));
            $procedure->addParam(new Param('P_CDPRODFIN',    Param::PARAM_INPUT, $row['PRODFIN']));
            $procedure->addParam(new Param('P_TPRODUTO',     Param::PARAM_INPUT, $row['TPRODUTO']));
            $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $row['CDFILIAL']));
            $procedure->addParam(new Param('P_CDMARCA',      Param::PARAM_INPUT, $row['CDMARCA']));
            $procedure->addParam(new Param('P_CDFORNECED',   Param::PARAM_INPUT, $row['CDFORNECED']));
            $procedure->addParam(new Param('P_CDTIPPART',    Param::PARAM_INPUT, $row['CDTIPPART']));
            $procedure->addParam(new Param('P_CDGRUPFILI',   Param::PARAM_INPUT, $row['CDGRUPFILI']));
            $procedure->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $row['CDOPERADOR']));
            $procedure->addParam(new Param('P_CHECK',        Param::PARAM_INPUT, $row['IDACAOEXPORT']));
            $procedure->addParam(new Param('P_CDFILIALORIG', Param::PARAM_INPUT, $row['CDFILIORIG']));
            $procedure->addParam(new Param('P_ROWSAFFECT',   Param::PARAM_OUTPUT, null));
            
            return $procedure->execute();
        } catch(Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
}
