<?php

namespace Teknisa\SUP\Modules\Cadastros\Services;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;

class FilialAlcadasService
{
    
    protected $connection;
    protected $newCode;
    
    public function __construct($entityManager, $newCode, $environment)
    {
        $this->connection  = $entityManager->getConnection();
        $this->newCode     = $newCode;
        $this->environment = $environment;
    }
    
    public function generateCode($newCode)
    {
        return $this->newCode->getCodeByTableName($newCode, $this->environment->getNrOrgTrab());
    }
    
    
    /********************
     *      INSERTS     *
     * ******************/

    public function saveFilialAlcada($codigoFilial, $numeroConfiguracaoAlcada, $consistirValorAprovacaoPor)
    {
        $sql = "INSERT INTO FILIALCADA (NRFILIALCADA, NRCONFIGALCADA, CDFILIAL, IDCONSISTVRAPROV, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU) VALUES (:NRFILIALCADA, :NRCONFIGALCADA, :CDFILIAL, :IDCONSISTVRAPROV, SYSDATE, :NRORGINCLUSAO, :CDOPERINCLUSAO, :NRORGULTATU, :CDOPERULTATU)";
        
        try {
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    'NRFILIALCADA'     => floatval(self::generateCode('NRFILIALCADA')),
                    'NRCONFIGALCADA'   => $numeroConfiguracaoAlcada,
                    'CDFILIAL'         => $codigoFilial,
                    'IDCONSISTVRAPROV' => $consistirValorAprovacaoPor,
                    'NRORGINCLUSAO'    => $this->environment->getNrOrg(),
                    'CDOPERINCLUSAO'   => $this->environment->getCdoperador(),
                    'NRORGULTATU'      => $this->environment->getNrOrg(),
                    'CDOPERULTATU'     => $this->environment->getCdoperador(),
                )
            );
        } catch (\Exception $e) {
            throw new \Exception('Erro ao cadastrar Unidade!');
        }
    }
    
    /********************
     *      UPDATES     *
     * ******************/

    public function updateFilialAlcada($params)
    {
        $sql = "UPDATE FILIALCADA SET
                        IDCONSISTVRAPROV = :P_IDCONSISTVRAPROV,
                        NRORGULTATU = :P_NRORGULTATU,
                        CDOPERULTATU = :P_CDOPERULTATU
                WHERE
                    NRFILIALCADA = :P_NRFILIALCADA
                    AND NRCONFIGALCADA = :P_NRCONFIGALCADA
                    AND CDFILIAL = :P_CDFILIAL";
        
        try {
            $std = $this->connection->prepare($sql);
            $std->execute(
                array(
                    'P_NRFILIALCADA'     => $params['NRFILIALCADA'],
                    'P_NRCONFIGALCADA'   => $params['NRCONFIGALCADA'],
                    'P_CDFILIAL'         => $params['CDFILIAL'],
                    'P_IDCONSISTVRAPROV' => $params['IDCONSISTVRAPROV'],
                    'P_NRORGULTATU'      => $this->environment->getNrOrg(),
                    'P_CDOPERULTATU'     => $this->environment->getCdoperador(),
                )
            );
        } catch (\Exception $e) {
            throw new \Exception('Erro ao editar Unidade!');
        }
    }
    
    
    /********************
     *      DELETES     *
     * ******************/
    
    public function deleteFilialAlcada($row)
    {
        try {
            $sql = "DELETE FROM FILIALCADA WHERE NRFILIALCADA = :NRFILIALCADA";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array (
                    'NRFILIALCADA' => $row['NRFILIALCADA'],
                )
            );
        } catch (Exception $e) {
            throw new \Exception('Erro ao deletar Unidade!');
        }
    }
    
    public function deleteTodasFiliaisAlcada($row)
    {
        try {
            $sql = "DELETE FROM FILIALCADA WHERE NRCONFIGALCADA = :NRCONFIGALCADA";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array (
                    'NRCONFIGALCADA' => $row['NRCONFIGALCADA'],
                )
            );
        } catch (Exception $e) {
            throw new \Exception('Erro ao deletar Unidades!');
        }
    }
    
    /********************
     *      FINDS       *
     * ******************/
    
    public function findFilialAlcada($NRFILIALCADA)
    {
        try {
            $sql = "SELECT F.NMFILIAL, FA.NRFILIALCADA, FA.CDFILIAL, FA.NRCONFIGALCADA, FA.NRORG, FA.DTINCLUSAO, FA.DTULTATU, FA.NRORGINCLUSAO, FA.CDOPERINCLUSAO, FA.NRORGULTATU, FA.CDOPERULTATU, ( SELECT COUNT(*) FROM APROVALCADA WHERE NRCONFIGALCADA = FA.NRCONFIGALCADA AND CDFILDEST = NVL( FA.CDFILIAL, CDFILDEST ) AND IDAPROVADO = 'PENDENTE') APROVACAO FROM FILIAL F JOIN FILIALCADA FA ON F.CDFILIAL = FA.CDFILIAL WHERE FA.NRFILIALCADA = :NRFILIALCADA";
            $std = $this->connection->prepare($sql);
            $std->execute(
                array (
                    'NRFILIALCADA' => $NRFILIALCADA,
                )
            );
            
            return $std->fetchAll()[0];
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function findAlcada($NRCONFIGALCADA)
    {
        $sql = "SELECT * FROM CONFIGALCADA WHERE NRCONFIGALCADA = :NRCONFIGALCADA";
        try {
            $std = $this->connection->prepare($sql);
            $std->execute(
                array (
                    'NRCONFIGALCADA' => $NRCONFIGALCADA,
                )
            );
            
            return $std->fetchAll()[0];
        } catch (Exception $e) {
            throw $e;
        }
    }
}