<?php
namespace Teknisa\SUP\Modules\Cotacao\Modules\ListaPreco\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Doctrine\ORM\EntityManager;

class Sup05500ListaPrecos extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = "sup05500_lista_precos";
    protected $entityManager;
    protected $crudService;
    protected $parameterBagColumns = array("CDFILIAL", "NRORG");
    protected $newCode;
    protected $dataSourceManager;
    
    public function __construct($crudService, ParameterBag $parameterBag, $environment, $connection, EntityManager $entityManager, $newCode, $dataSourceManager) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment = $environment;
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->newCode = $newCode;
        $this->dataSourceManager    = $dataSourceManager;
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterCriteria = $request->getFilterCriteria();
            $filterCriteria->setDataSourceName($this->dataSourceName);
            $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
            $filterCriteria->addCondition("CDFILIAL", "=", $this->getCDFILCENTCMP());
            $newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, $this->parameterBagColumns);
            $this->crudService->find($response, $newFilterCriteria);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function getCDFILCENTCMP() {
        $query = "SELECT NVL(CDFILCENTCMP, CDFILIAL) AS CDFILCENTCMP FROM FILIAL WHERE (CDFILIAL = :CDFILIAL)";
        $stmt = $this->connection->prepare($query);
        $stmt->execute(array(
            "CDFILIAL" => $this->environment->getCdFilial()
        ));
        return $stmt->fetchAll()[0]['CDFILCENTCMP'];
    }
    
    public function disponibilizar (DTO\Request $request, DTO\Response $response) {
        
        $data = $request->getDataSet()->getRows();
        $cdfilial = $this->environment->getCdFilial();
        $cdoperador = $this->environment->getCdOperador();
        $cdfilcentcmp = $this->getCDFILCENTCMP();
        
        for($i=0;$i<sizeof($data);$i++){
            $query = "UPDATE PREVLISTA SET IDDISPLISFIL = 'S' WHERE CDFILIAL = :CDFILIAL AND NRSEQPRELIST = :NRSEQPRELIST";
            $stmt = $this->connection->prepare($query);
            $stmt->execute(array(
			    "CDFILIAL" => $cdfilcentcmp,
			    "NRSEQPRELIST" => $data[$i]['NRSEQPRELIST']
			));
			
			$query = "UPDATE ITEMCOLE SET IDPRECALT = 'S' WHERE CDFILIAL = :CDFILIAL AND NRSEQPRELIST = :NRSEQPRELIST";
            $stmt = $this->connection->prepare($query);
            $stmt->execute(array(
			    "CDFILIAL" => $cdfilcentcmp,
			    "NRSEQPRELIST" => $data[$i]['NRSEQPRELIST']
			));
			
			$query = "INSERT INTO LOGDISPLISTA (CDCENTCMPLDL, NRLISTPRELDL, CDOPERATULDL, DTATUACUSLDL) VALUES (:CDCENTCMPLDL, :NRLISTPRELDL, :CDOPERATULDL, SYSDATE)";
            $stmt = $this->connection->prepare($query);
            $stmt->execute(array(
			    "CDCENTCMPLDL" => $cdfilcentcmp,
			    "NRLISTPRELDL" => $data[$i]['NRSEQPRELIST'],
			    "CDOPERATULDL" => $cdoperador
			));
			
        }
    }
    public function buscaTodasRegioes($nrorg) {
        $reg = array();
        
        $conditions = array(
            [
                "columnName" => "NRORG",
                "operator"   => "=",
                "value"      => $nrorg
            ]
        );
        
        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('regiao', $conditions);
        $listaRegiao = $this->dataSourceManager->findBy($filterCriteria)->getRows();

        foreach($listaRegiao as $regiao){
            array_push($reg, $regiao['CDREGIAO']);
          
        }
         return $reg;
    }
    public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        );
        
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    public function atualizar (DTO\Request $request, DTO\Response $response) {
			
        $data = $request->getDataSet()->getRows();
        $cdfilial = $this->environment->getCdFilial();
        $cdoperador = $this->environment->getCdOperador();
        $cdfilcentcmp = $this->getCDFILCENTCMP();
        $nrorg = $this->environment->getNrOrg();
         
			
		for ($i=0;$i<sizeof($data);$i++) {
            $NRLISTA = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
			$query = "INSERT INTO LOGDISPCUSTLIST (CDCENTCMPLDISP, NRLISTPRELDISP, CDOPERATULDISP, DTATUACUSLDISP) VALUES (:CDCENTCMPLDISP, :NRLISTPRELDISP, :CDOPERATULDISP, SYSDATE)";
            $stmt = $this->connection->prepare($query);
            $stmt->execute(array(
                "CDCENTCMPLDISP" => $cdfilcentcmp,
                "NRLISTPRELDISP" => $data[$i]['NRSEQPRELIST'],
                "CDOPERATULDISP" => $cdoperador
			));
			
			$query = "INSERT INTO LOGCUSTLISTA (CDCENTCMPLAC, NRLISTPRELAC, CDTELAATULAC, CDOPERATULAC, CDFILIATULAC, DTINICUSLAC) VALUES (:CDCENTCMPLAC, :NRLISTPRELAC, 'SUP05500', :CDOPERATULAC, NULL, SYSDATE)";
            $stmt = $this->connection->prepare($query);
            $stmt->execute(array(
                "CDCENTCMPLAC" => $cdfilcentcmp,
                "NRLISTPRELAC" => $data[$i]['NRSEQPRELIST'],
                "CDOPERATULAC" => $cdoperador
			));
			
            if($data[$i]['ALLCDREGIAO'] == 'T'){
                $regioes = $this->buscaTodasRegioes($nrorg);
            }
            else{
                $regioes = array();
                $regioes = $data[$i]['CDREGIAO'];
               
                $regioes = explode("_", $data[$i]['CDREGIAO']);
            }
           
            foreach($regioes as $regiao) {
               
                $this ->insertLista($NRLISTA, $regiao);
                 
            }
            $lista = $NRLISTA;
  
			if ($data[$i]['IDPREVSELLIS']==='N'||$data[$i]['IDPREVSELLIS']==='') {
			     
			    $this->LISPREC_UPPRODFILI($cdfilcentcmp, $data[$i]['NRSEQPRELIST'], $lista);
			  
			} else {
			    $this->ATUALIZA_CUSTO_ITEMCOL($cdfilcentcmp, $data[$i]['NRSEQPRELIST']);
			}
			
			$query = "UPDATE LOGCUSTLISTA SET DTATUACUSLAC = SYSDATE WHERE CDCENTCMPLAC = :CDCENTCMPLAC AND NRLISTPRELAC = :NRLISTPRELAC AND CDTELAATULAC = 'SUP05500' AND CDOPERATULAC = :CDOPERATULAC AND CDFILIATULAC IS NULL AND DTINICUSLAC = (SELECT MAX(DTINICUSLAC) FROM LOGCUSTLISTA L WHERE CDCENTCMPLAC = :CDCENTCMPLAC AND NRLISTPRELAC = :NRLISTPRELAC AND CDTELAATULAC = 'SUP05500' AND CDOPERATULAC = :CDOPERATULAC AND CDFILIATULAC IS NULL)";
            $stmt = $this->connection->prepare($query);
            $stmt->execute(array(
			    "CDCENTCMPLAC" => $cdfilcentcmp,
			    "NRLISTPRELAC" => $data[$i]['NRSEQPRELIST'],
			    "CDOPERATULAC" => $cdoperador
			));
			
			$query = "UPDATE PREVLISTA SET IDATUCUSLIST = 'N', IDPREVSELLIS = 'S' WHERE CDFILIAL = :CDFILIAL AND NRSEQPRELIST = :NRSEQPRELIST";
            $stmt = $this->connection->prepare($query);
            $stmt->execute(array(
			    "CDFILIAL" => $cdfilcentcmp,
			    "NRSEQPRELIST" => $data[$i]['NRSEQPRELIST']
			));
		}
		
    }
    public function LISPREC_UPPRODFILI($cdfilial, $nrseqprelist, $lista) {
        
        $connection = $this->entityManager->getConnection();
        $procedure = new StoredProcedure($connection, 'LISPREC_UPPRODFILI');
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSEQPRELIST', Param::PARAM_INPUT, $nrseqprelist, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ATUCUSTO', Param::PARAM_INPUT, 'N', Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILCUSTO', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDLISTAREG', Param::PARAM_INPUT, $lista, Param::PARAM_TYPE_STR));
           
        $procedure->execute();

    }
    
    public function ATUALIZA_CUSTO_ITEMCOL($cdfilial, $nrseqprelist) {
        $connection = $this->entityManager->getConnection();
        $procedure = new StoredProcedure($connection, 'ATUALIZA_CUSTO_ITEMCOL');
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSEQPRELIST', Param::PARAM_INPUT, $nrseqprelist, Param::PARAM_TYPE_STR));
        $procedure->execute();
       
    }
}