<?php

namespace Teknisa\SUP\Services;

use Teknisa\SUP\Util\SupUtilities;

class ProductsAFService
{

    const CDFILLANC = '9999';
    const CDMARCA = '00680';

    public function __construct(SupUtilities $supUtilities)
    {
        $this->supUtilities = $supUtilities;
    }

    public function getProduct($cdProduct)
    {
        $query = "SELECT CDPRODUTO, CDARVPROD, NMPRODUTO FROM PRODUTO WHERE CDPRODUTO = :P_CDPRODUTO";
        $params = array( "P_CDPRODUTO" => $cdProduct );
        return $this->supUtilities->crud($query, $params);
    }

    public function getProductToSupplier($cdSupplier, $cdProduct)
    {
        $query = "SELECT MAX(CDPRODCOMPL) CDPRODCOMPL FROM PRODFORNCOMP WHERE CDFORNECED = :P_CDFORNECED AND :P_CDPRODUTO LIKE CDPRODUTO||'%'";
        $params = array(
            'P_CDFORNECED' => $cdSupplier,
            'P_CDPRODUTO' => $cdProduct,
        );
        return $this->supUtilities->crud($query, $params);
    }

    public function getBestPrice($cdSupplier, $cdFilDest, $cdProduct, $orderDate, $centralCmp)
    {
        $query = "SELECT FILIAL.CDFILIAL, NVL(ITEMCOLE.VRPRECUNIITM, 0) VRPRECUNIITM FROM ITEMCOLE, FILIAL, PRODUTO WHERE ITEMCOLE.CDFILIAL = :P_CDCENTCMP AND ITEMCOLE.CDFORNECED = :P_CDFORNECED AND FILIAL.CDFILIAL = :P_CDFILDEST AND PRODUTO.CDPRODUTO = :P_CDPRODUTO AND ITEMCOLE.CDREGIAO = FILIAL.CDREGIAO AND ITEMCOLE.CDPRODUTO = PRODUTO.CDPRODUTO AND NVL(ITEMCOLE.VRPRECUNIITM, 0) > 0 AND TO_DATE(:P_DTPROGENTAF, 'DD/MM/YYYY') BETWEEN ITEMCOLE.DTINIVALITM AND ITEMCOLE.DTFINVALITM";
        $params = array(
            'P_CDCENTCMP' => $centralCmp,
            'P_CDFORNECED' => $cdSupplier,
            'P_CDFILDEST' => $cdFilDest,
            'P_DTPROGENTAF' => $orderDate,
            'P_CDPRODUTO' => $cdProduct,
        );
        return $this->supUtilities->crud($query, $params);
    }

    public function getCentralCmp($cdFilial)
    {
        $query = "SELECT NVL(CDFILCENTCMP, CDFILIAL) AS CDFILCENTCMP FROM FILIAL WHERE CDFILIAL = :P_CDFILIAL";
        $param = array( 'P_CDFILIAL' => $cdFilial );
        return $this->supUtilities->crud($query, $param)[0];
    }

    public function setProduct($product, $dataToSave)
    {
        $query = "INSERT INTO ITEMAUTF (CDFILDEST, CDFILLANC, NRAUTOFORN, CDPRODUTO, QTPROGENTAF, VRPREUNIAF, DTPROGENTAF, CDMARCA) SELECT :P_CDFILDEST, :P_CDFILLANC, :P_NRAUTOFORN, :P_CDPRODUTO, :P_QTPROGENTAF, :P_VRPREUNIAF, :P_DTPROGENTAF, :P_CDMARCA FROM DUAL WHERE NOT EXISTS(SELECT 1 FROM ITEMAUTF I WHERE I.CDFILDEST = :P_CDFILDEST AND I.CDFILLANC = :P_CDFILLANC AND I.NRAUTOFORN = :P_NRAUTOFORN AND I.CDPRODUTO = :P_CDPRODUTO AND I.DTPROGENTAF = :P_DTPROGENTAF AND I.CDMARCA = :P_CDMARCA)";
        $params = array(
            'P_CDFILDEST' => $dataToSave['cdFildest'],
            'P_CDFILLANC' => self::CDFILLANC,
            'P_NRAUTOFORN' => $dataToSave['nrAutoForn'],
            'P_CDPRODUTO' => (string) $product->{'codcliente'},
            'P_QTPROGENTAF' => (string) $product->{'quantidade'},
            'P_VRPREUNIAF' => (float) str_replace(',', '.', (string) $product->{'vlr-unitario'}),
            'P_DTPROGENTAF' => $dataToSave['orderDate'],
            'P_CDMARCA' => self::CDMARCA,
        );

        $this->supUtilities->crud($query, $params);
    }

}
